#******************************************
# Email2IM Version 1
# Written by Suramya Tomar (suramya@suramya.com)
# http://www.suramya.com
# Last updated: 3/27/2010
#
# Released Under the GPL Ver 2.
#
# This program is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or    
# (at your option) any later version.     
#         
# This program is distributed in the hope that it will be useful, but  
# WITHOUT ANY WARRANTY; without even the implied warranty of   
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  
# General Public License for more details.    
#         
# You should have received a copy of the GNU General Public License  
# along with this program; if not, write to the    
# Free Software Foundation, Inc.,      
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   
#########################################################################

#!/usr/bin/perl

use strict;
use warnings;
no warnings 'redefine';

use Net::POP3;
use Net::XMPP;
use XML::Stream;

# *******************************************************
# User defined variables for Google Talk Account access *
# *******************************************************


my $gTalk_UserName  = "YOUR GTALK Account Username";		# This is the account that is used to send the IM's
my $gTalk_Password  = "YOUR GTALK Account Password";
my $resource  = "Email2IM v1.0";

# ************************************************
# User defined variables for Email server access *
# ************************************************

 my $ServerName = "YOUR POP3 Server";			# Any emails sent to this email account will be sent out as IM's
 my $UserName = "YOUR EMAIL USERNAME";
 my $Password = "YOUR EMAIL PASSWORD";
 
 # Other Variables

 my $pop3;
 my $Num_Messages;
 my $Messages;
 my $msg_id;
 my $MsgContent;
 my $message;
 my $recipient;

# ************************************************************************************
# GetEmail()																		 *
#																					 *
# Connects to a POP3 email server and waits for an incoming email. It then downloads *
# the email for further processing, which is done in ProcessEmail()					 *
# ************************************************************************************

sub GetEmail()
{
	 while(0==0)												# The program will run in an infinite loop.
	 {
		print "Waiting for email...\n";							# Print status message

		$pop3 = Net::POP3->new($ServerName, Port => '110');					# Connect to email server
		   die "Couldn't log on to server" unless $pop3;

		$Num_Messages = $pop3->login($UserName, $Password);	# Login to the server and get No of messages
		  die "Couldn't log on to server" unless $pop3;

		$Messages = $pop3->list();							# Get message list
		
		foreach $msg_id (keys(%$Messages))
		{
			$MsgContent = $pop3->top($msg_id, 40);
			ProcessEmail(@$MsgContent);
			$pop3->delete ($msg_id);
		}

		# Close the connection
		$pop3->quit();
		sleep(20);				# Sleep for 20 seconds
	 }
}

# **********************************************************************************
# This subroutine parses the 'From', 'Subject' and Body fields of a message header *
# Then calls the SendGtalkIM subroutine to send the actual IM out                  *
# **********************************************************************************

sub ProcessEmail
{
  # Assign parameter to a local variable
  my (@lines) = @_;
  my $body_start =0;
  $message = "";

  # Declare local variables
  my ($from, $line);

  # Check each line in the header
  foreach $line (@lines)
  {
    if($line =~ m/^From: (.*)/)
    {
       # We found the "From" field, so let's get what we need
       $from = $1;
       $from =~ s/"|<.*>//g;
       $from = substr($from, 0, 39);				# This gives us the 'From' Name
   }
   elsif( $line =~ m/^Subject: (.*)/)
   {
       # We found the "Subject" field. This contains the No to send the SMS to.

	   $recipient   = $1;
   }
   elsif(( $line =~ m/^Envelope-To:/)||($body_start eq 1)) # This is the last line in the email header
   {                                                                           # after this the body starts
	   if($body_start ne 0)
	   {
		   $message = $message . $line;
	   }

	   $body_start=1;
   }
  }

  # At this point we know the Subject, From and Body.
  # So we can send the Message out to the specified IM name

  $message = $message . "\n\n IM Sent via Email2IM (http://scripts.suramya.com/Email2IM/)";
  SendGtalkIM();
}

# ***************************************************************************************************
# This function connects to the Google Talk Server and sends the IM to the Recipient defined above. *
# ***************************************************************************************************

sub SendGtalkIM
{
	my $client = Net::XMPP::Client->new;

	my $status = $client->Connect(
									hostname => 'talk.google.com',
									port => 5222,
									componentname => 'gmail.com',
									connectiontype => 'tcpip',
									tls => 1,);

	if (!defined $status) 
	{
		die "Connection failed: $!"
	}


	my ($res,$msg) = $client->AuthSend(
										username => $gTalk_UserName,
										password => $gTalk_Password,
										resource => $resource, # client name
	);

	die "Auth failed ",
		defined $msg ? $msg : '',
		" $!"
		unless defined $res and $res eq 'ok';


	$client->MessageSend(
		   to => $recipient, 
		   body => $message,
		   resource => $resource,
		   subject => 'message via ' . $resource,
		   );
} # End of the SendIM function


# Patching XML::Stream to support the google-added JID
# Thanks to http://blogs.perl.org/users/marco_fontani/2010/03/google-talk-with-perl.html
# for this work around/fix. 

sub SASLAuth 
{
	my $self = shift;
	my $sid  = shift;
	my $first_step = $self->{SIDS}->{$sid}->{sasl}->{client}->client_start();
	my $first_step64 = MIME::Base64::encode_base64($first_step,"");
	$self->Send( $sid,
				"<auth xmlns='" . &ConstXMLNS('xmpp-sasl') .
				"' mechanism='" .
				$self->{SIDS}->{$sid}->{sasl}->{client}->mechanism() .
				"' " .
				q{xmlns:ga='http://www.google.com/talk/protocol/auth'
				ga:client-uses-full-bind-result='true'} . # JID
				">".$first_step64."</auth>");
}
